<?php
/**
* NOTICE OF LICENSE
*
* This file is licenced under the Software License Agreement.
* With the purchase or the installation of the software in your application
* you accept the licence agreement.
*
* You must not modify, adapt or create derivative works of this source code
*
*  @author    AvoSMS
*  @copyright 2020-2022 AvoSMS
*  @license   LICENSE.txt
*/
if (!defined('_PS_VERSION_')) {
    exit;
}

class AvoSMS extends Module
{
    private static $var_empty;

    public function __construct()
    {
        $this->name = 'avosms';
        $this->module_key = '79b402044e516b1218bd7cfb67dc9afd';
        $this->tab = 'advertising_marketing';
        $this->version = '1.0.0';
        $this->author = 'AvoSMS';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = [
            'min' => '1.6.0',
            'max' => '1.7.9',
        ];
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('AvoSMS');
        $this->description = $this->l('Automate the sending of SMS to your customers at important stages of their order. Send your marketing campaigns and SMS in a few clicks individually or for a specific segment. Interconnect your Prestashop shop to our SMS sending platform easily.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall this module?');

        if (!Configuration::get('MYMODULE_NAME')) {
            $this->warning = $this->l('No name defined for this module.');
        }
    }

    public function installTabParent($class_name, $name)
    {
        $tab = new Tab();
        foreach (Language::getLanguages(false) as $lang) {
            $tab->name[$lang['id_lang']] = $this->l($name);
        }
        $tab->module = $this->name;
        $tab->class_name = $class_name;
        $tab->id_parent = 0;
        return $tab->add();
    }

    public function installTab($parent_class, $class_name, $name, $iconInfo)
    {
        $tab = new Tab();
        foreach (Language::getLanguages(false) as $lang) {
            $tab->name[$lang['id_lang']] = $this->l($name);
        }
        $tab->class_name = $class_name;
        $tab->id_parent = Tab::getIdFromClassName($parent_class);
        $tab->module = $this->name;
        $tab->icon = $iconInfo;
        return $tab->add();
    }

    public function hookActionDispatcher($args)
    {
        if (defined('_PS_ADMIN_DIR_')) {
            if (isset($args['controller_class']) && $args['controller_class'] == 'AdminModulesController') {
                if (Tools::getValue('configure') == 'avosms') {
                    $link = new Link();
                    Tools::redirectAdmin($link->getAdminLink('ConfigAvoSMS'));
                }
            }
        }
    }

    public function install()
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        include dirname(__FILE__) . '/sql/install.php';

        return
            parent::install()
            && $this->registerHook('actionValidateOrder')
            && $this->registerHook('actionOrderStatusPostUpdate')
            && $this->registerHook('actionDispatcher')
            && $this->installTabParent('AdminAvoSMS', 'AvoSMS')
            && $this->installTab('AdminAvoSMS', 'SendSmsAvoSMS', 'Send SMS', 'sms')
            && $this->installTab('AdminAvoSMS', 'ConfigAvoSMS', 'Configuration', 'settings')
            && Configuration::updateValue('MYMODULE_NAME', 'AvoSMS');
    }

    public function uninstall()
    {
        include dirname(__FILE__) . '/sql/uninstall.php';

        return
            parent::uninstall()
            && Configuration::deleteByName('AvoSMS');
    }

    public function getContent()
    {
        return true;
    }

    public function queryValueBdd($name)
    {
        /**
         * Permet de check si le name exist
         */
        $query = 'SELECT * FROM ' . _DB_PREFIX_ . 'avosms_configuration WHERE name = "' . $name . '"';
        $query_exec = Db::getInstance()->ExecuteS($query);
        $nb_result_query = count($query_exec);

        if ($nb_result_query == 0) {
            $value = '';
        } else {
            $value = $query_exec[0]['value'];
        }

        return $value;
    }

    public function hookActionOrderStatusPostUpdate(array $params)
    {
        $contenu_sms_send = AvoSMS::$var_empty;
        $prenom_customer = AvoSMS::$var_empty;
        $nom_customer = AvoSMS::$var_empty;
        $reference_commande = AvoSMS::$var_empty;
        $phone = AvoSMS::$var_empty;
        $is_sms_to_send = 0;

        $name_shop = Configuration::get('PS_SHOP_NAME');

        $id_new_status_order = $params['newOrderStatus']->id;

        if ($id_new_status_order == 4 || $id_new_status_order == 6) {
            $id_order = $params['id_order'];

            /**
             * Permet de récupérer la commande
             */
            $order = new Order($id_order);

            $id_adresse_delivery = $order->id_address_delivery;
            $id_customer = $order->id_customer;
            $reference_commande = $order->reference;

            /**
             * Permet de récupérer le customer
             */
            $customer = new Customer($id_customer);

            $prenom_customer = $customer->firstname;
            $nom_customer = $customer->lastname;

            /**
             * Permet de récupérer l'adresse
             */
            $address = new Address($id_adresse_delivery);

            $phone = $address->phone;

            if (!empty($phone) && $phone != null) {
                if ($id_new_status_order == 4) {
                    $commande_expedie_actived = $this->queryValueBdd('AVOSMS_COMMANDE_EXPEDIE_CHECKBOX');

                    if ($commande_expedie_actived == 1) {
                        $contenu_sms_send = $this->queryValueBdd('AVOSMS_COMMANDE_EXPEDIE_CONTENU');
                        $is_sms_to_send = 1;
                    }
                } elseif ($id_new_status_order == 6) {
                    $commande_cancel_actived = $this->queryValueBdd('AVOSMS_COMMANDE_ANNULE_CHECKBOX');

                    if ($commande_cancel_actived == 1) {
                        $contenu_sms_send = $this->queryValueBdd('AVOSMS_COMMANDE_ANNULE_CONTENU');
                        $is_sms_to_send = 1;
                    }
                }
            }
        }

        if ($is_sms_to_send == 1) {
            /* Gestion des variables personnalisé du contenu */
            if (stristr($contenu_sms_send, '%prenomclient%')) {
                $contenu_sms_send = str_replace('%prenomclient%', $prenom_customer, $contenu_sms_send);
            } elseif (stristr($contenu_sms_send, '%customerfirstname%')) {
                $contenu_sms_send = str_replace('%customerfirstname%', $prenom_customer, $contenu_sms_send);
            }

            if (stristr($contenu_sms_send, '%nomclient%')) {
                $contenu_sms_send = str_replace('%nomclient%', $nom_customer, $contenu_sms_send);
            } elseif (stristr($contenu_sms_send, '%customerlastname%')) {
                $contenu_sms_send = str_replace('%customerlastname%', $nom_customer, $contenu_sms_send);
            }

            if (stristr($contenu_sms_send, '%numerocommande%')) {
                $contenu_sms_send = str_replace('%numerocommande%', $reference_commande, $contenu_sms_send);
            } elseif (stristr($contenu_sms_send, '%ordernumber%')) {
                $contenu_sms_send = str_replace('%ordernumber%', $reference_commande, $contenu_sms_send);
            }

            if (stristr($contenu_sms_send, '%nomboutique%')) {
                $contenu_sms_send = str_replace('%nomboutique%', $name_shop, $contenu_sms_send);
            } elseif (stristr($contenu_sms_send, '%shopname%')) {
                $contenu_sms_send = str_replace('%shopname%', $name_shop, $contenu_sms_send);
            }

            $identifiant_api = $this->queryValueBdd('AVOSMS_IDENTIFIANT_API');
            $cle_api = $this->queryValueBdd('AVOSMS_CLE_API');

            $nom_expediteur = $this->queryValueBdd('AVOSMS_NOM_EXPEDITEUR');

            if (empty($nom_expediteur) || $nom_expediteur == null) {
                $nom_expediteur = '';
            }

            $curl = curl_init('https://api.avosms.com/v1/sms/send');
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_COOKIESESSION, true);

            $postfields = [
                'id' => $identifiant_api,
                'key' => $cle_api,
                'sender' => $nom_expediteur,
                'recipients' => [$phone],
                'message' => $contenu_sms_send,
                'type' => 'N',
            ];

            $postfields_json = json_encode($postfields);
            $trame = http_build_query($postfields, '', '&');
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $trame);
            $raw_response = curl_exec($curl);
            curl_close($curl);

            $response = json_decode($raw_response);

            $code_erreur = $response->code;

            if ($code_erreur == 'OK') {
                $id_envoi_sms = $response->send_id;
            }
        }
    }

    /**
     * Hook Action Validation Commande
     */
    public function hookActionValidateOrder(array $params)
    {
        $name_shop = Configuration::get('PS_SHOP_NAME');

        $id_adresse_delivery = $params['order']->id_address_delivery;
        $id_address_invoice = $params['order']->id_address_invoice;
        $reference_commande = $params['order']->reference;
        $id_customer = $params['customer']->id;

        /**
         * Permet de récupérer le customer
         */
        $customer = new Customer($id_customer);

        $prenom_customer = $customer->firstname;
        $nom_customer = $customer->lastname;

        /**
         * Permet de récupérer l'adresse
         */
        $address = new Address($id_adresse_delivery);

        $phone = $address->phone;

        if (!empty($phone) && $phone != null) {
            $commande_confirm_actived = $this->queryValueBdd('AVOSMS_COMMANDE_CONFIRM_CHECKBOX');

            if ($commande_confirm_actived == 1) {
                $identifiant_api = $this->queryValueBdd('AVOSMS_IDENTIFIANT_API');
                $cle_api = $this->queryValueBdd('AVOSMS_CLE_API');

                $contenu_sms_commande_confirm = $this->queryValueBdd('AVOSMS_COMMANDE_CONFIRM_CONTENU');

                $nom_expediteur = $this->queryValueBdd('AVOSMS_NOM_EXPEDITEUR');

                if (empty($nom_expediteur) || $nom_expediteur == null) {
                    $nom_expediteur = '';
                }

                /* Gestion des variables personnalisé du contenu */
                if (stristr($contenu_sms_commande_confirm, '%prenomclient%')) {
                    $contenu_sms_commande_confirm = str_replace('%prenomclient%', $prenom_customer, $contenu_sms_commande_confirm);
                } elseif (stristr($contenu_sms_commande_confirm, '%customerfirstname%')) {
                    $contenu_sms_commande_confirm = str_replace('%customerfirstname%', $prenom_customer, $contenu_sms_commande_confirm);
                }

                if (stristr($contenu_sms_commande_confirm, '%nomclient%')) {
                    $contenu_sms_commande_confirm = str_replace('%nomclient%', $nom_customer, $contenu_sms_commande_confirm);
                } elseif (stristr($contenu_sms_commande_confirm, '%customerlastname%')) {
                    $contenu_sms_commande_confirm = str_replace('%customerlastname%', $nom_customer, $contenu_sms_commande_confirm);
                }

                if (stristr($contenu_sms_commande_confirm, '%numerocommande%')) {
                    $contenu_sms_commande_confirm = str_replace('%numerocommande%', $reference_commande, $contenu_sms_commande_confirm);
                } elseif (stristr($contenu_sms_commande_confirm, '%ordernumber%')) {
                    $contenu_sms_commande_confirm = str_replace('%ordernumber%', $reference_commande, $contenu_sms_commande_confirm);
                }

                if (stristr($contenu_sms_commande_confirm, '%nomboutique%')) {
                    $contenu_sms_commande_confirm = str_replace('%nomboutique%', $name_shop, $contenu_sms_commande_confirm);
                } elseif (stristr($contenu_sms_commande_confirm, '%shopname%')) {
                    $contenu_sms_commande_confirm = str_replace('%shopname%', $name_shop, $contenu_sms_commande_confirm);
                }

                $curl = curl_init('https://api.avosms.com/v1/sms/send');
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($curl, CURLOPT_COOKIESESSION, true);

                $postfields = [
                    'id' => $identifiant_api,
                    'key' => $cle_api,
                    'sender' => $nom_expediteur,
                    'recipients' => [$phone],
                    'message' => $contenu_sms_commande_confirm,
                    'type' => 'N',
                ];

                $postfields_json = json_encode($postfields);
                $trame = http_build_query($postfields, '', '&');
                curl_setopt($curl, CURLOPT_POST, true);
                curl_setopt($curl, CURLOPT_POSTFIELDS, $trame);
                $raw_response = curl_exec($curl);
                curl_close($curl);

                $response = json_decode($raw_response);

                $code_erreur = $response->code;

                if ($code_erreur == 'OK') {
                    $id_envoi_sms = $response->send_id;
                }
            }
        }

        $commande_confirm_gerant_actived = $this->queryValueBdd('AVOSMS_COMMANDE_CONFIRM_GERANT_CHECKBOX');

        if ($commande_confirm_gerant_actived == 1) {
            $identifiant_api = $this->queryValueBdd('AVOSMS_IDENTIFIANT_API');
            $cle_api = $this->queryValueBdd('AVOSMS_CLE_API');

            $numero_telephone_gerant = $this->queryValueBdd('AVOSMS_COMMANDE_CONFIRM_GERANT_TELEPHONE');
            $contenu_sms_commande_confirm_gerant = $this->queryValueBdd('AVOSMS_COMMANDE_CONFIRM_GERANT_CONTENU');

            $nom_expediteur = $this->queryValueBdd('AVOSMS_NOM_EXPEDITEUR');

            if (empty($nom_expediteur) || $nom_expediteur == null) {
                $nom_expediteur = '';
            }

            /* Gestion des variables personnalisé du contenu */
            if (stristr($contenu_sms_commande_confirm_gerant, '%prenomclient%')) {
                $contenu_sms_commande_confirm_gerant = str_replace('%prenomclient%', $prenom_customer, $contenu_sms_commande_confirm_gerant);
            } elseif (stristr($contenu_sms_commande_confirm_gerant, '%customerfirstname%')) {
                $contenu_sms_commande_confirm_gerant = str_replace('%customerfirstname%', $prenom_customer, $contenu_sms_commande_confirm_gerant);
            }

            if (stristr($contenu_sms_commande_confirm_gerant, '%nomclient%')) {
                $contenu_sms_commande_confirm_gerant = str_replace('%nomclient%', $nom_customer, $contenu_sms_commande_confirm_gerant);
            } elseif (stristr($contenu_sms_commande_confirm_gerant, '%customerlastname%')) {
                $contenu_sms_commande_confirm_gerant = str_replace('%customerlastname%', $nom_customer, $contenu_sms_commande_confirm_gerant);
            }

            if (stristr($contenu_sms_commande_confirm_gerant, '%numerocommande%')) {
                $contenu_sms_commande_confirm_gerant = str_replace('%numerocommande%', $reference_commande, $contenu_sms_commande_confirm_gerant);
            } elseif (stristr($contenu_sms_commande_confirm_gerant, '%ordernumber%')) {
                $contenu_sms_commande_confirm_gerant = str_replace('%ordernumber%', $reference_commande, $contenu_sms_commande_confirm_gerant);
            }

            if (stristr($contenu_sms_commande_confirm_gerant, '%nomboutique%')) {
                $contenu_sms_commande_confirm_gerant = str_replace('%nomboutique%', $name_shop, $contenu_sms_commande_confirm_gerant);
            } elseif (stristr($contenu_sms_commande_confirm_gerant, '%shopname%')) {
                $contenu_sms_commande_confirm_gerant = str_replace('%shopname%', $name_shop, $contenu_sms_commande_confirm_gerant);
            }

            $curl = curl_init('https://api.avosms.com/v1/sms/send');
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_COOKIESESSION, true);

            $postfields = [
                'id' => $identifiant_api,
                'key' => $cle_api,
                'sender' => $nom_expediteur,
                'recipients' => [$numero_telephone_gerant],
                'message' => $contenu_sms_commande_confirm_gerant,
                'type' => 'N',
            ];

            $postfields_json = json_encode($postfields);
            $trame = http_build_query($postfields, '', '&');
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $trame);
            $raw_response = curl_exec($curl);
            curl_close($curl);

            $response = json_decode($raw_response);

            $code_erreur = $response->code;

            if ($code_erreur == 'OK') {
                $id_envoi_sms = $response->send_id;
            }
        }
    }
}
