<?php
/**
* NOTICE OF LICENSE
*
* This file is licenced under the Software License Agreement.
* With the purchase or the installation of the software in your application
* you accept the licence agreement.
*
* You must not modify, adapt or create derivative works of this source code
*
*  @author    AvoSMS
*  @copyright 2020-2022 AvoSMS
*  @license   LICENSE.txt
*/
class ConfigAvoSMSController extends ModuleAdminController
{
    public function __construct()
    {
        // $this->name = 'formConfigAvosms';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function renderList()
    {
        $output = '';

        $identifiant_api = $this->queryValueBdd('AVOSMS_IDENTIFIANT_API');
        $cle_api = $this->queryValueBdd('AVOSMS_CLE_API');

        if (empty($identifiant_api) && empty($cle_api)) {
            $open_link_inscription = '<a href="https://www.avosms.com/inscription" target="_blank">';
            $open_link_connexion = '<a href="https://app.avosms.com" target="_blank">';
            $close_link = '</a>';

            $phrase_inscription_connexion = sprintf(
                $this->l('%sCreate your account%s or %slog in%s to your customer area'),
                $open_link_inscription,
                $close_link,
                $open_link_connexion,
                $close_link,
            );

            $output .= '<div class="panel">
                                <div class="panel-heading">
                                    ' . $this->l('Configuration help') . '
                                </div>
                                <div class="h4">
                                    ' . $this->l('You are only a few steps away from completing the integration of our SMS sending module.') . '<br/>
                                    ' . $this->l('In order to finalise the installation, please follow the procedures below :') . '<br/><br/>

                                    <li>' . $phrase_inscription_connexion . '</li>
                                    <li>' . $this->l('Go to the "API Access" page') . '</li>
                                    <li>' . $this->l('Retrieve your API access information and enter it in the fields below') . '</li>

                                    <br/><br/>
                                    ' . $this->l('Once these steps are completed, the module will be connected to your AvoSMS account and you will have access to all features.') . '
                                </div>
                            </div>';
        }

        $output .= $this->displayFormConfig();

        if (!empty($identifiant_api) && !empty($cle_api)) {
            $output .= $this->afficheInfoConfig();
        }

        return $output;
    }

    public function postProcess()
    {
        $isErreur = 0;

        if (Tools::isSubmit('submitConfigAvosms')) {
            $identifiant_api = (string) Tools::getValue('AVOSMS_IDENTIFIANT_API');
            $cle_api = (string) Tools::getValue('AVOSMS_CLE_API');

            if (empty($identifiant_api) || !Validate::isGenericName($identifiant_api)) {
                $this->errors[] = $this->l('Please enter your API ID.');
            } elseif (empty($cle_api) || !Validate::isGenericName($cle_api)) {
                $this->errors[] = $this->l('Please enter your API key.');
            } else {
                if (!$this->updateBdd('AVOSMS_IDENTIFIANT_API', $identifiant_api)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_CLE_API', $cle_api)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if ($isErreur == 0) {
                    $this->confirmations[] = $this->l('The changes have been successfully implemented.');
                }
            }
        }

        if (Tools::isSubmit('submitConfigGeneraleAvosms')) {
            $nom_expediteur = (string) Tools::getValue('AVOSMS_NOM_EXPEDITEUR');

            $suivi_clic = (string) Tools::getValue('AVOSMS_SUIVI_CLIC_CHECKBOX');

            $commande_confirm_gerant_checkbox = (string) Tools::getValue('AVOSMS_COMMANDE_CONFIRM_GERANT_CHECKBOX');
            $commande_confirm_gerant_telephone = (string) Tools::getValue('AVOSMS_COMMANDE_CONFIRM_GERANT_TELEPHONE');
            $commande_confirm_gerant_contenu = (string) Tools::getValue('AVOSMS_COMMANDE_CONFIRM_GERANT_CONTENU');

            $commande_confirm_checkbox = (string) Tools::getValue('AVOSMS_COMMANDE_CONFIRM_CHECKBOX');
            $commande_confirm_contenu = (string) Tools::getValue('AVOSMS_COMMANDE_CONFIRM_CONTENU');

            $commande_expedie_checkbox = (string) Tools::getValue('AVOSMS_COMMANDE_EXPEDIE_CHECKBOX');
            $commande_expedie_contenu = (string) Tools::getValue('AVOSMS_COMMANDE_EXPEDIE_CONTENU');

            $commande_annule_checkbox = (string) Tools::getValue('AVOSMS_COMMANDE_ANNULE_CHECKBOX');
            $commande_annule_contenu = (string) Tools::getValue('AVOSMS_COMMANDE_ANNULE_CONTENU');

            if (empty($nom_expediteur) || !Validate::isGenericName($nom_expediteur)) {
                $this->errors[] = $this->l('The sender name must be between 3 and 11 characters long. Check your input.');
            } elseif ($commande_confirm_gerant_checkbox == 1 && empty($commande_confirm_gerant_telephone)) {
                $this->errors[] = $this->l('Please enter the manager\'s phone number to send an SMS when an order is confirmed.');
            } elseif ($commande_confirm_gerant_checkbox == 1 && empty($commande_confirm_gerant_contenu)) {
                $this->errors[] = $this->l('Please fill in the content of the SMS for the confirmation of an order to the manager.');
            } elseif ($commande_confirm_checkbox == 1 && empty($commande_confirm_contenu)) {
                $this->errors[] = $this->l('Please fill in the content of the SMS for the confirmation of an order.');
            } elseif ($commande_expedie_checkbox == 1 && empty($commande_expedie_contenu)) {
                $this->errors[] = $this->l('Please fill in the content of the SMS for the shipment of an order.');
            } elseif ($commande_annule_checkbox == 1 && empty($commande_annule_contenu)) {
                $this->errors[] = $this->l('Please fill in the content of the SMS for the cancellation of an order.');
            } else {
                if (!$this->updateBdd('AVOSMS_NOM_EXPEDITEUR', $nom_expediteur)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_SUIVI_CLIC_CHECKBOX', $suivi_clic)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_COMMANDE_CONFIRM_GERANT_CHECKBOX', $commande_confirm_gerant_checkbox)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_COMMANDE_CONFIRM_GERANT_TELEPHONE', $commande_confirm_gerant_telephone)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_COMMANDE_CONFIRM_GERANT_CONTENU', $commande_confirm_gerant_contenu)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_COMMANDE_CONFIRM_CHECKBOX', $commande_confirm_checkbox)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_COMMANDE_CONFIRM_CONTENU', $commande_confirm_contenu)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_COMMANDE_EXPEDIE_CHECKBOX', $commande_expedie_checkbox)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_COMMANDE_EXPEDIE_CONTENU', $commande_expedie_contenu)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_COMMANDE_ANNULE_CHECKBOX', $commande_annule_checkbox)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if (!$this->updateBdd('AVOSMS_COMMANDE_ANNULE_CONTENU', $commande_annule_contenu)) {
                    $this->errors[] = $this->l('An error has occurred.');
                    $isErreur = 1;
                }

                if ($isErreur == 0) {
                    $this->confirmations[] = $this->l('The changes have been successfully implemented.');
                }
            }
        }
        return parent::postProcess();
    }

    public function updateBdd($name, $value)
    {
        /**
         * Permet de check si le name exist
         */
        $query = 'SELECT * FROM ' . _DB_PREFIX_ . 'avosms_configuration WHERE name = "' . $name . '"';
        $query_exec = Db::getInstance()->ExecuteS($query);
        $nb_result_query = count($query_exec);

        if ($nb_result_query == 0) {
            /**
             * Permet de faire l'insert de la configuration
             */
            $date_now_ins = date('Y-m-d H:i:s');

            if (!Db::getInstance()->insert('avosms_configuration', ['name' => $name, 'value' => $value, 'date_create' => $date_now_ins])) {
                return false;
            }

            return true;
        } else {
            /* Permet de faire l'insert de la configuration */
            if (!Db::getInstance()->update('avosms_configuration', ['value' => $value], 'name = "' . $name . '"')) {
                return false;
            }

            return true;
        }
    }

    public function queryValueBdd($name)
    {
        /**
         * Permet de check si le name exist
         */
        $query = 'SELECT * FROM ' . _DB_PREFIX_ . 'avosms_configuration WHERE name = "' . $name . '"';
        $query_exec = Db::getInstance()->ExecuteS($query);
        $nb_result_query = count($query_exec);

        if ($nb_result_query == 0) {
            $value = '';
        } else {
            $value = $query_exec[0]['value'];
        }

        return $value;
    }

    public function displayFormConfig()
    {
        $form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Configure'),
                ],
                'input' => [
                    [
                        'type' => 'text',
                        'label' => $this->l('API ID'),
                        'name' => 'AVOSMS_IDENTIFIANT_API',
                        'maxlength' => 150,
                        'required' => true,
                    ],

                    [
                        'type' => 'text',
                        'label' => $this->l('API Key'),
                        'name' => 'AVOSMS_CLE_API',
                        'maxlength' => 150,
                        'required' => true,
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Confirm'),
                    'class' => 'btn btn-default pull-right',
                ],
            ],
        ];

        $helper = new HelperForm();
        $helper->show_toolbar = true;
        $helper->table = $this->table;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitConfigAvosms';
        $helper->currentIndex = $this->context->link->getAdminLink('ConfigAvoSMS', false);
        $helper->token = Tools::getAdminToken($this->controller_name . (int) $this->id . (int) $this->context->employee->id);
        $helper->fields_value['AVOSMS_IDENTIFIANT_API'] = $this->queryValueBdd('AVOSMS_IDENTIFIANT_API');
        $helper->fields_value['AVOSMS_CLE_API'] = $this->queryValueBdd('AVOSMS_CLE_API');

        return $helper->generateForm([$form]);
    }

    public function displayFormConfigGenerale()
    {
        /**
         * Nom expéditeur
         */
        $avosms_nom_expediteur = $this->queryValueBdd('AVOSMS_NOM_EXPEDITEUR');

        $this->context->smarty->assign('avosms_nom_expediteur', $avosms_nom_expediteur);

        /**
         * Suivi des clics
         */
        $suivi_clic_checkbox = $this->queryValueBdd('AVOSMS_SUIVI_CLIC_CHECKBOX');

        if ($suivi_clic_checkbox == 1) {
            $avosms_checked_suivi_clic = 'checked';
        } else {
            $avosms_checked_suivi_clic = '';
        }

        $this->context->smarty->assign('avosms_checked_suivi_clic', $avosms_checked_suivi_clic);

        /**
         * Commande confirmation gérant
         */
        $commande_confirm_gerant_checkbox = $this->queryValueBdd('AVOSMS_COMMANDE_CONFIRM_GERANT_CHECKBOX');

        if ($commande_confirm_gerant_checkbox == 1) {
            $avosms_checked_commande_confirm_gerant = 'checked';
        } else {
            $avosms_checked_commande_confirm_gerant = '';
        }

        $avosms_commande_confirm_gerant_telephone = $this->queryValueBdd('AVOSMS_COMMANDE_CONFIRM_GERANT_TELEPHONE');
        $avosms_commande_confirm_gerant_contenu = $this->queryValueBdd('AVOSMS_COMMANDE_CONFIRM_GERANT_CONTENU');

        $this->context->smarty->assign('avosms_checked_commande_confirm_gerant', $avosms_checked_commande_confirm_gerant);
        $this->context->smarty->assign('avosms_commande_confirm_gerant_telephone', $avosms_commande_confirm_gerant_telephone);
        $this->context->smarty->assign('avosms_commande_confirm_gerant_contenu', $avosms_commande_confirm_gerant_contenu);

        /**
         * Commande confirmation
         */
        $commande_confirm_checkbox = $this->queryValueBdd('AVOSMS_COMMANDE_CONFIRM_CHECKBOX');

        if ($commande_confirm_checkbox == 1) {
            $avosms_checked_commande_confirm = 'checked';
        } else {
            $avosms_checked_commande_confirm = '';
        }

        $avosms_commande_confirm_contenu = $this->queryValueBdd('AVOSMS_COMMANDE_CONFIRM_CONTENU');

        $this->context->smarty->assign('avosms_checked_commande_confirm', $avosms_checked_commande_confirm);
        $this->context->smarty->assign('avosms_commande_confirm_contenu', $avosms_commande_confirm_contenu);

        /**
         * Commande expédiée
         */
        $commande_expedie_checkbox = $this->queryValueBdd('AVOSMS_COMMANDE_EXPEDIE_CHECKBOX');

        if ($commande_expedie_checkbox == 1) {
            $avosms_checked_commande_expedie = 'checked';
        } else {
            $avosms_checked_commande_expedie = '';
        }

        $avosms_commande_expedie_contenu = $this->queryValueBdd('AVOSMS_COMMANDE_EXPEDIE_CONTENU');

        $this->context->smarty->assign('avosms_checked_commande_expedie', $avosms_checked_commande_expedie);
        $this->context->smarty->assign('avosms_commande_expedie_contenu', $avosms_commande_expedie_contenu);

        /**
         * Commande annulée
         */
        $commande_annule_checkbox = $this->queryValueBdd('AVOSMS_COMMANDE_ANNULE_CHECKBOX');

        if ($commande_annule_checkbox == 1) {
            $avosms_checked_commande_annule = 'checked';
        } else {
            $avosms_checked_commande_annule = '';
        }

        $avosms_commande_annule_contenu = $this->queryValueBdd('AVOSMS_COMMANDE_ANNULE_CONTENU');

        $this->context->smarty->assign('avosms_checked_commande_annule', $avosms_checked_commande_annule);
        $this->context->smarty->assign('avosms_commande_annule_contenu', $avosms_commande_annule_contenu);

        $output = $this->module->display(_PS_MODULE_DIR_ . 'avosms', 'views/templates/admin/configAvoSMS.tpl');

        return $output;
    }

    public function afficheInfoConfig()
    {
        $identifiant_api = $this->queryValueBdd('AVOSMS_IDENTIFIANT_API');
        $cle_api = $this->queryValueBdd('AVOSMS_CLE_API');

        $curl = curl_init('https://api.avosms.com/v1/account/balance');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_COOKIESESSION, true);

        $postfields = [
            'id' => $identifiant_api,
            'key' => $cle_api,
        ];

        $postfields_json = json_encode($postfields);
        $trame = http_build_query($postfields, '', '&');
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $trame);
        $raw_response = curl_exec($curl);
        curl_close($curl);

        $response = json_decode($raw_response);

        $status_response = $response->status;
        $code = $response->code;

        if ($code == 'OK') {
            $balance = $response->balance;

            $text_balance = sprintf(
                $this->l('%s credit(s)'),
                $balance
            );

            $output = '<div class="panel">
                                <div class="panel-heading">
                                    ' . $this->l('My AvoSMS balance') . '
                                </div>
                                <div class="text-center">
                                    <h4>' . $text_balance . '</h4>
                                </div>
                            </div>';

            $output .= $this->displayFormConfigGenerale();
        } else {
            $open_link = '<a href="https://app.avosms.com" target="_blank">';
            $close_link = '</a>';

            $last_phrase = sprintf(
                $this->l('To retrieve your identifiers, go to your %scustomer area%s, "API Access" section'),
                $open_link,
                $close_link
            );

            $output = '<div class="panel">
                            <div class="text-center h4">
                                ' . $this->l('Your AvoSMS "API Access" information is incorrect. Please check these and try again.') . '
                                <br/><br/>
                                ' . $this->l('If you think this is a mistake, please contact our services.') . '
                                <br/><br/>
                                ' . $last_phrase . '
                            </div>
                        </div>';
        }

        return $output;
    }
}
