<?php
/**
* NOTICE OF LICENSE
*
* This file is licenced under the Software License Agreement.
* With the purchase or the installation of the software in your application
* you accept the licence agreement.
*
* You must not modify, adapt or create derivative works of this source code
*
*  @author    AvoSMS
*  @copyright 2020-2022 AvoSMS
*  @license   LICENSE.txt
*/
class SendSmsAvoSMSController extends ModuleAdminController
{
    public function __construct()
    {
        // $this->name = 'formSendSmsAvosms';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function renderList()
    {
        $output = '';

        $identifiant_api = $this->queryValueBdd('AVOSMS_IDENTIFIANT_API');
        $cle_api = $this->queryValueBdd('AVOSMS_CLE_API');

        if (empty($identifiant_api) && empty($cle_api)) {
            $open_link_inscription = '<a href="https://www.avosms.com/inscription" target="_blank">';
            $open_link_connexion = '<a href="https://app.avosms.com" target="_blank">';
            $close_link = '</a>';

            $phrase_inscription_connexion = sprintf(
                $this->l('%sCreate your account%s or %slog in%s to your customer area'),
                $open_link_inscription,
                $close_link,
                $open_link_connexion,
                $close_link,
            );

            $output .= '<div class="panel">
                                <div class="panel-heading">
                                    ' . $this->l('Configuration help') . '
                                </div>
                                <div class="h4">
                                    ' . $this->l('You are only a few steps away from completing the integration of our SMS sending module.') . '<br/>
                                    ' . $this->l('In order to finalise the installation, please follow the procedures below :') . '<br/><br/>

                                    <li>' . $phrase_inscription_connexion . '</li>
                                    <li>' . $this->l('Go to the "API Access" page') . '</li>
                                    <li>' . $this->l('Retrieve your API access information and enter it into the fields on the configuration page') . '</li>

                                    <br/><br/>
                                    ' . $this->l('Once these steps are completed, the module will be connected to your AvoSMS account and you will have access to all features.') . '
                                </div>
                            </div>';
        } else {
            $output .= $this->afficheInfoConfig();
        }

        return $output;
    }

    public function postProcess()
    {
        $isErreur = 0;

        if (Tools::isSubmit('submitSendSmsAvosms')) {
            /**
             *Contenu du SMS
             */
            $send_sms_contenu_sms = (string) Tools::getValue('AVOSMS_SEND_SMS_CONTENU');

            if (empty($send_sms_contenu_sms)) {
                $this->errors[] = $this->l('Please fill in the content of the SMS.');
                return;
            }

            /**
             * Mode Envoi
             */
            $send_sms_mode_envoi = (string) Tools::getValue('AVOSMS_SEND_SMS_MODE_ENVOI');

            if (empty($send_sms_mode_envoi)) {
                $this->errors[] = $this->l('Please select the delivery method.');
                return;
            }

            /**
             * Gestion Moment Envoi
             */
            $send_sms_moment_envoi = (string) Tools::getValue('AVOSMS_SEND_SMS_MOMENT_ENVOI');

            if (strlen($send_sms_moment_envoi) != 16) {
                $this->errors[] = $this->l('The sending date is not correctly formatted.');
                return;
            }

            $send_sms_moment_envoi_explode = explode(' ', $send_sms_moment_envoi);

            $send_sms_moment_envoi_date = $send_sms_moment_envoi_explode[0];
            $send_sms_moment_envoi_time = $send_sms_moment_envoi_explode[1];

            $identifiant_api = $this->queryValueBdd('AVOSMS_IDENTIFIANT_API');
            $cle_api = $this->queryValueBdd('AVOSMS_CLE_API');

            /**
             * Gestion liste destinataire
             */
            $recipients = [];

            if ($send_sms_mode_envoi == 1) {
                /**
                 * Formatter liste destinataire - Client
                 */
                $query_customer_list = 'SELECT * FROM ' . _DB_PREFIX_ . 'address WHERE active = 1 AND deleted = 0 AND phone != "" AND id_customer IN (SELECT id_customer FROM ' . _DB_PREFIX_ . 'customer WHERE active = 1 AND deleted = 0) GROUP BY phone;';
                $query_customer_list_exec = Db::getInstance()->ExecuteS($query_customer_list);

                foreach ($query_customer_list_exec as $result_customer_list) {
                    $phone_customer = $result_customer_list['phone'];

                    $recipients[] = $phone_customer;
                }
            } elseif ($send_sms_mode_envoi == 2) {
                /**
                 * Formatter liste destinataire - Client Newsletter
                 */
                $query_customer_abo_newsletter_list = 'SELECT * FROM ' . _DB_PREFIX_ . 'address WHERE active = 1 AND deleted = 0 AND phone != "" AND id_customer IN (SELECT id_customer FROM ' . _DB_PREFIX_ . 'customer WHERE active = 1 AND deleted = 0 AND newsletter = 1) GROUP BY phone;';
                $query_customer_abo_newsletter_list_exec = Db::getInstance()->ExecuteS($query_customer_abo_newsletter_list);

                foreach ($query_customer_abo_newsletter_list_exec as $result_customer_abo_newsletter_list) {
                    $phone_customer_abo_newsletter = $result_customer_abo_newsletter_list['phone'];

                    $recipients[] = $phone_customer_abo_newsletter;
                }
            } elseif ($send_sms_mode_envoi == 3) {
                /**
                 * Formatter liste destinataire - Liste Contact AvoSMS
                 */
                $send_sms_liste_destinataire = (string) Tools::getValue('AVOSMS_SEND_SMS_LISTE_CONTACT');

                if ($send_sms_liste_destinataire == 0) {
                    $this->errors[] = $this->l('Please select the AvoSMS contact list.');
                    return;
                }

                $curl = curl_init('https://api.avosms.com/v1/contact/list/information');
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($curl, CURLOPT_COOKIESESSION, true);

                $postfields = [
                    'id' => $identifiant_api,
                    'key' => $cle_api,
                    'list_contact_id' => $send_sms_liste_destinataire,
                ];

                $postfields_json = json_encode($postfields);
                $trame = http_build_query($postfields, '', '&');
                curl_setopt($curl, CURLOPT_POST, true);
                curl_setopt($curl, CURLOPT_POSTFIELDS, $trame);
                $raw_response = curl_exec($curl);
                curl_close($curl);

                $response = json_decode($raw_response);

                $status_response = $response->status;
                $code = $response->code;

                if ($code == 'OK') {
                    $contactList = $response->list_contact;

                    foreach ($contactList as $contact_list) {
                        $contact_telephone_number = $contact_list->contact_telephone_number;

                        $recipients[] = $contact_telephone_number;
                    }
                }
            } elseif ($send_sms_mode_envoi == 4) {
                /**
                 * Formatter liste destinataire - Destinataire unique
                 */
                $send_sms_numero_destinataire = (string) Tools::getValue('AVOSMS_SEND_SMS_NUMERO');

                if (empty($send_sms_numero_destinataire)) {
                    $this->errors[] = $this->l('Please enter the recipient\'s number');
                    return;
                }

                $recipients[] = $send_sms_numero_destinataire;
            }

            if (count($recipients) == 0) {
                $this->errors[] = $this->l('No recipient to send the SMS to.');
                return;
            }

            /**
             * Permet d'envoyer le SMS
             */
            $nom_expediteur = $this->queryValueBdd('AVOSMS_NOM_EXPEDITEUR');

            if (empty($nom_expediteur) || $nom_expediteur == null) {
                $nom_expediteur = '';
            }

            $suivi_clic = $this->queryValueBdd('AVOSMS_SUIVI_CLIC_CHECKBOX');

            if (empty($suivi_clic)) {
                $suivi_clic = 0;
            }

            $curl = curl_init('https://api.avosms.com/v1/sms/send');
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_COOKIESESSION, true);

            $postfields = [
                'id' => $identifiant_api,
                'key' => $cle_api,
                'sender' => $nom_expediteur,
                'recipients' => $recipients,
                'message' => $send_sms_contenu_sms,
                'deliveryDate' => $send_sms_moment_envoi_date,
                'deliveryHour' => $send_sms_moment_envoi_time,
                'type' => 'M',
                'clickFollow' => $suivi_clic,
            ];

            $postfields_json = json_encode($postfields);
            $trame = http_build_query($postfields, '', '&');
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $trame);
            $raw_response = curl_exec($curl);
            curl_close($curl);

            $response = json_decode($raw_response);

            $code_erreur = $response->code;

            if ($code_erreur == 'OK') {
                $id_envoi_sms = $response->send_id;
                $this->confirmations[] = $this->l('The SMS was successfully sent!');
            } else {
                $message_error_with_code = sprintf(
                    $this->l('An error occurred during the sending (%s).'),
                    $code_erreur,
                );
                $this->errors[] = $message_error_with_code;
                return;
            }
        }

        return parent::postProcess();
    }

    public function queryValueBdd($name)
    {
        /**
         * Permet de check si le name exist
         */
        $query = 'SELECT * FROM ' . _DB_PREFIX_ . 'avosms_configuration WHERE name = "' . $name . '"';
        $query_exec = Db::getInstance()->ExecuteS($query);
        $nb_result_query = count($query_exec);

        if ($nb_result_query == 0) {
            $value = '';
        } else {
            $value = $query_exec[0]['value'];
        }

        return $value;
    }

    public function displayFormSendSms()
    {
        $this->addCSS(_THEME_CSS_DIR_ . 'jquery-ui-timepicker-addon.css');
        $this->addJqueryUi(['ui.datepicker', 'ui.slider']);
        $this->addJS([_THEME_JS_DIR_ . 'jquery-ui-timepicker-addon.js']);

        $date_now = date('d/m/Y H:i');
        $this->context->smarty->assign('dateNow', $date_now);

        /**
         * Permet de récupérer les listes de contacts AvoSMS
         */
        $identifiant_api = $this->queryValueBdd('AVOSMS_IDENTIFIANT_API');
        $cle_api = $this->queryValueBdd('AVOSMS_CLE_API');

        $curl = curl_init('https://api.avosms.com/v1/contact/list');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_COOKIESESSION, true);

        $postfields = [
            'id' => $identifiant_api,
            'key' => $cle_api,
        ];

        $postfields_json = json_encode($postfields);
        $trame = http_build_query($postfields, '', '&');
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $trame);
        $raw_response = curl_exec($curl);
        curl_close($curl);

        $response = json_decode($raw_response);

        $status_response = $response->status;
        $code = $response->code;

        if ($code == 'OK') {
            $list_contact = $response->list_contact;
            $nb_liste_contact = count($list_contact);

            if ($nb_liste_contact == 0) {
                $affiche_liste_contact_avosms = '<div class="alert alert-info">' . $this->l('No AvoSMS contact list registered on your account. Please login to your AvoSMS customer area to create your first contact list') . '</div>';
            } else {
                $affiche_liste_contact_avosms = '<label class="control-label col-lg-4 required text-right" for="AVOSMS_SEND_SMS_LISTE_CONTACT"> ' . $this->l('AvoSMS contact list') . '</label>
                                                <div class="col-lg-8">
                                                    <select name="AVOSMS_SEND_SMS_LISTE_CONTACT" id="AVOSMS_SEND_SMS_LISTE_CONTACT">
                                                        <option value="0">' . $this->l('Select a contact list') . '</option>';

                foreach ($list_contact as $contactList) {
                    $list_contact_id = $contactList->list_contact_id;
                    $list_contact_name = $contactList->list_contact_name;
                    $nb_contact_list = $contactList->nb_contact_list;

                    $affiche_liste_contact_avosms .= '<option value="' . $list_contact_id . '">' . $list_contact_name . ' - ' . $nb_contact_list . ' ' . $this->l('recipient(s)') . '</option>';
                }

                $affiche_liste_contact_avosms .= '</select>
                                                </div>';
            }
        } else {
            $message_error_get_list_contact = sprintf(
                $this->l('An error occurred while retrieving your AvoSMS contact list (%s).'),
                $code
            );

            $affiche_liste_contact_avosms = '<div class="alert alert-danger">' . $message_error_get_list_contact . '</div>';
        }

        $this->context->smarty->assign('affiche_liste_contact_avosms', $affiche_liste_contact_avosms);

        /**
         * Permet de récupérer les modèles AvoSMS
         */
        $curl = curl_init('https://api.avosms.com/v1/model/sms/list');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_COOKIESESSION, true);

        $postfields = [
            'id' => $identifiant_api,
            'key' => $cle_api,
        ];

        $postfields_json = json_encode($postfields);
        $trame = http_build_query($postfields, '', '&');
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $trame);
        $raw_response = curl_exec($curl);
        curl_close($curl);

        $response = json_decode($raw_response);

        $status_response = $response->status;
        $code = $response->code;

        if ($code == 'OK') {
            $list_modele_sms = $response->list_model_sms;
            $nb_modele = count($list_modele_sms);

            if ($nb_modele == 0) {
                $affiche_liste_modele_avosms = '';
            } else {
                $affiche_liste_modele_avosms = '<div class="form-group">
                                                    <label class="control-label col-lg-4 text-right" for="AVOSMS_SEND_SMS_MODELE"> ' . $this->l('AvoSMS models') . '</label>
                                                    <div class="col-lg-8">
                                                        <select name="AVOSMS_SEND_SMS_MODELE" id="AVOSMS_SEND_SMS_MODELE">
                                                            <option value="0">' . $this->l('Select your desired SMS model') . '</option>';

                foreach ($list_modele_sms as $model) {
                    $model_id = $model->model_id;
                    $model_name = $model->model_name;
                    $model_content = $model->model_content;

                    $affiche_liste_modele_avosms .= '<option value="' . $model_id . '" data-content="' . $model_content . '">' . $model_name . '</option>';
                }

                $affiche_liste_modele_avosms .= '</select>
                                                </div>
                                            </div>';
            }
        } else {
            $message_error_get_list_model = sprintf(
                $this->l('An error occurred while retrieving your AvoSMS models (%s).'),
                $code
            );

            $affiche_liste_modele_avosms = '<div class="alert alert-danger">' . $message_error_get_list_model . '</div>';
        }

        $this->context->smarty->assign('affiche_liste_modele_avosms', $affiche_liste_modele_avosms);

        /**
         * Récupération du nombre de client
         */
        $query_nb_customer = 'SELECT * FROM ' . _DB_PREFIX_ . 'address WHERE active = 1 AND deleted = 0 AND phone != "" AND id_customer IN (SELECT id_customer FROM ' . _DB_PREFIX_ . 'customer WHERE active = 1 AND deleted = 0) GROUP BY phone;';
        $query_nb_customer_exec = Db::getInstance()->ExecuteS($query_nb_customer);
        $nb_result_query_nb_customer = count($query_nb_customer_exec);

        if ($nb_result_query_nb_customer <= 1) {
            $avosms_send_sms_affiche_nb_client = $nb_result_query_nb_customer . ' ' . $this->l('customer');
        } else {
            $avosms_send_sms_affiche_nb_client = $nb_result_query_nb_customer . ' ' . $this->l('customers');
        }

        $this->context->smarty->assign('avosms_send_sms_affiche_nb_client', $avosms_send_sms_affiche_nb_client);

        /**
         * Récupération du nombre de client abonné à la newsletter
         */
        $query_nb_customer_abo_newsletter = 'SELECT * FROM ' . _DB_PREFIX_ . 'address WHERE active = 1 AND deleted = 0 AND phone != "" AND id_customer IN (SELECT id_customer FROM ' . _DB_PREFIX_ . 'customer WHERE active = 1 AND deleted = 0 AND newsletter = 1) GROUP BY phone;';
        $query_nb_customer_abo_newsletter_exec = Db::getInstance()->ExecuteS($query_nb_customer_abo_newsletter);
        $nb_result_query_nb_customer_abo_newsletter = count($query_nb_customer_abo_newsletter_exec);

        if ($nb_result_query_nb_customer_abo_newsletter <= 1) {
            $avosms_send_sms_affiche_nb_client_abo_newsletter = $nb_result_query_nb_customer_abo_newsletter . ' ' . $this->l('customer');
        } else {
            $avosms_send_sms_affiche_nb_client_abo_newsletter = $nb_result_query_nb_customer_abo_newsletter . ' ' . $this->l('customers');
        }

        $this->context->smarty->assign('avosms_send_sms_affiche_nb_client_abo_newsletter', $avosms_send_sms_affiche_nb_client_abo_newsletter);

        /**
         * Affichage du formulaire
         */
        $output = $this->module->display(_PS_MODULE_DIR_ . 'avosms', 'views/templates/admin/form_send_sms_avosms.tpl');

        return $output;
    }

    public function afficheInfoConfig()
    {
        $identifiant_api = $this->queryValueBdd('AVOSMS_IDENTIFIANT_API');
        $cle_api = $this->queryValueBdd('AVOSMS_CLE_API');

        $curl = curl_init('https://api.avosms.com/v1/account/balance');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_COOKIESESSION, true);

        $postfields = [
            'id' => $identifiant_api,
            'key' => $cle_api,
        ];

        $postfields_json = json_encode($postfields);
        $trame = http_build_query($postfields, '', '&');
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $trame);
        $raw_response = curl_exec($curl);
        curl_close($curl);

        $response = json_decode($raw_response);

        $status_response = $response->status;
        $code = $response->code;

        if ($code == 'OK') {
            $balance = $response->balance;
            $output = $this->displayFormSendSms();
        } else {
            $open_link = '<a href="https://app.avosms.com" target="_blank">';
            $close_link = '</a>';

            $last_phrase = sprintf(
                $this->l('To retrieve your identifiers, go to your %scustomer area%s, "API Access" section'),
                $open_link,
                $close_link
            );

            $output = '<div class="panel">
                            <div class="text-center h4">
                                ' . $this->l('Your AvoSMS "API Access" information is incorrect. Please check these and try again.') . '
                                <br/><br/>
                                ' . $this->l('If you think this is a mistake, please contact our services.') . '
                                <br/><br/>
                                ' . $last_phrase . '
                            </div>
                        </div>';
        }

        return $output;
    }
}
